/* -----------------------------------------------------------------*-C-*-
   ffitarget.h - Copyright (c) 1996-2003  Red Hat, Inc.
#if (defined(__ppc__) || defined(__ppc64__))
   Copyright (C) 2007, 2008 Free Software Foundation, Inc
   Target configuration macros for PowerPC.
#else /* ! __ppc__ && ! __ppc64__ */
   Copyright (C) 2008  Free Software Foundation, Inc.

   Target configuration macros for x86 and x86-64.
#endif /* __ppc__ || __ppc64__ */

   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
   ``Software''), to deal in the Software without restriction, including
   without limitation the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED ``AS IS'', WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
   DEALINGS IN THE SOFTWARE.

   ----------------------------------------------------------------------- */

#ifndef LIBFFI_TARGET_H
#define LIBFFI_TARGET_H

/* ---- System specific configurations ----------------------------------- */

#if (defined(__ppc__) || defined(__ppc64__))
#if defined (POWERPC) && defined (__powerpc64__)	/* linux64 */
#define POWERPC64
#elif defined (POWERPC_DARWIN) && defined (__ppc64__)	/* Darwin */
#define POWERPC64
#elif defined (POWERPC_AIX) && defined (__64BIT__)	/* AIX64 */
#define POWERPC64
#else /* ! __ppc__ && ! __ppc64__ */
#if defined (X86_64) && defined (__i386__)
#undef X86_64
#define X86
#endif

#ifdef X86_WIN64
#define FFI_SIZEOF_ARG 8
#define USE_BUILTIN_FFS 0 // not yet implemented in mingw-64 
#endif /* __ppc__ || __ppc64__ */
#endif

#if defined (__i386__) || defined(__x86_64__)
/* ---- Generic type definitions ----------------------------------------- */

#endif /* __i386__ || __x86_64__ */
#ifndef LIBFFI_ASM
#if defined (__i386__) || defined(__x86_64__)
#ifdef X86_WIN64
#ifdef _MSC_VER
typedef unsigned __int64       ffi_arg;
typedef __int64                ffi_sarg;
#else
typedef unsigned long long     ffi_arg;
typedef long long              ffi_sarg;
#endif
#else
#endif /* __i386__ || __x86_64__ */
typedef unsigned long          ffi_arg;
typedef signed long            ffi_sarg;
#if defined (__i386__) || defined(__x86_64__)
#endif
#endif /* __i386__ || __x86_64__ */

typedef enum ffi_abi {
  FFI_FIRST_ABI = 0,

#if (defined(__ppc__) || defined(__ppc64__))
#ifdef POWERPC
#else /* ! __ppc__ && ! __ppc64__ */
  /* ---- Intel x86 Win32 ---------- */
#ifdef X86_WIN32
#endif /* __ppc__ || __ppc64__ */
  FFI_SYSV,
#if (defined(__ppc__) || defined(__ppc64__))
  FFI_GCC_SYSV,
  FFI_LINUX64,
  FFI_LINUX,
  FFI_LINUX_SOFT_FLOAT,
# ifdef POWERPC64
  FFI_DEFAULT_ABI = FFI_LINUX64,
# else
#  if (!defined(__NO_FPRS__) && (__LDBL_MANT_DIG__ == 106))
  FFI_DEFAULT_ABI = FFI_LINUX,
#  else
#   ifdef __NO_FPRS__
  FFI_DEFAULT_ABI = FFI_LINUX_SOFT_FLOAT,
#   else
  FFI_DEFAULT_ABI = FFI_GCC_SYSV,
#   endif
#  endif
# endif
#endif

#ifdef POWERPC_AIX
  FFI_AIX,
  FFI_DARWIN,
  FFI_DEFAULT_ABI = FFI_AIX,
#else /* ! __ppc__ && ! __ppc64__ */
  FFI_STDCALL,
  /* TODO: Add fastcall support for the sake of completeness */
  FFI_DEFAULT_ABI = FFI_SYSV,
#endif /* __ppc__ || __ppc64__ */
#endif

#if (defined(__ppc__) || defined(__ppc64__))
#ifdef POWERPC_DARWIN
  FFI_AIX,
  FFI_DARWIN,
  FFI_DEFAULT_ABI = FFI_DARWIN,
#endif
#else /* ! __ppc__ && ! __ppc64__ */
#ifdef X86_WIN64
  FFI_WIN64,
  FFI_DEFAULT_ABI = FFI_WIN64,
#else
#endif /* __ppc__ || __ppc64__ */

#if (defined(__ppc__) || defined(__ppc64__))
#ifdef POWERPC_FREEBSD
#else /* ! __ppc__ && ! __ppc64__ */
  /* ---- Intel x86 and AMD x86-64 - */
#if !defined(X86_WIN32) && (defined(__i386__) || defined(__x86_64__))
#endif /* __ppc__ || __ppc64__ */
  FFI_SYSV,
#if (defined(__ppc__) || defined(__ppc64__))
  FFI_GCC_SYSV,
  FFI_LINUX64,
  FFI_LINUX,
  FFI_LINUX_SOFT_FLOAT,
#else /* ! __ppc__ && ! __ppc64__ */
  FFI_UNIX64,   /* Unix variants all use the same ABI for x86-64  */
#ifdef __i386__
#endif /* __ppc__ || __ppc64__ */
  FFI_DEFAULT_ABI = FFI_SYSV,
#if defined (__i386__) || defined(__x86_64__)
#else
  FFI_DEFAULT_ABI = FFI_UNIX64,
#endif
#endif /* __i386__ || __x86_64__ */
#endif
#if defined (__i386__) || defined(__x86_64__)
#endif /* X86_WIN64 */
#endif /* __i386__ || __x86_64__ */

#if (defined(__ppc__) || defined(__ppc64__))
  FFI_LAST_ABI
#else /* ! __ppc__ && ! __ppc64__ */
  FFI_LAST_ABI = FFI_DEFAULT_ABI + 1
#endif /* __ppc__ || __ppc64__ */
} ffi_abi;
#endif

/* ---- Definitions for closures ----------------------------------------- */

#define FFI_CLOSURES 1
#if (defined(__ppc__) || defined(__ppc64__))
#define FFI_NATIVE_RAW_API 0

/* For additional types like the below, take care about the order in
   ppc_closures.S. They must follow after the FFI_TYPE_LAST.  */

/* Needed for soft-float long-double-128 support.  */
#define FFI_TYPE_UINT128 (FFI_TYPE_LAST + 1)

/* Needed for FFI_SYSV small structure returns.
   We use two flag bits, (FLAG_SYSV_SMST_R3, FLAG_SYSV_SMST_R4) which are
   defined in ffi.c, to determine the exact return type and its size.  */
#define FFI_SYSV_TYPE_SMALL_STRUCT (FFI_TYPE_LAST + 2)
#else /* ! __ppc__ && ! __ppc64__ */
#define FFI_TYPE_SMALL_STRUCT_1B (FFI_TYPE_LAST + 1)
#define FFI_TYPE_SMALL_STRUCT_2B (FFI_TYPE_LAST + 2)
#define FFI_TYPE_SMALL_STRUCT_4B (FFI_TYPE_LAST + 3)
#endif /* __ppc__ || __ppc64__ */

#if (defined(__ppc__) || defined(__ppc64__))
#if defined(POWERPC64) || defined(POWERPC_AIX)
#else /* ! __ppc__ && ! __ppc64__ */
#if defined (X86_64) || (defined (__x86_64__) && defined (X86_DARWIN))
#endif /* __ppc__ || __ppc64__ */
#define FFI_TRAMPOLINE_SIZE 24
#if (defined(__ppc__) || defined(__ppc64__))
#else /* POWERPC || POWERPC_AIX */
#define FFI_TRAMPOLINE_SIZE 40
#else /* ! __ppc__ && ! __ppc64__ */
#define FFI_NATIVE_RAW_API 0
#else
#ifdef X86_WIN32
#define FFI_TRAMPOLINE_SIZE 13
#else
#ifdef X86_WIN64
#define FFI_TRAMPOLINE_SIZE 29
#define FFI_NATIVE_RAW_API 0
#define FFI_NO_RAW_API 1
#else
#define FFI_TRAMPOLINE_SIZE 10
#endif /* __ppc__ || __ppc64__ */
#endif
#if (defined(__ppc__) || defined(__ppc64__))

#ifndef LIBFFI_ASM
#if defined(POWERPC_DARWIN) || defined(POWERPC_AIX)
struct ffi_aix_trampoline_struct {
    void * code_pointer;	/* Pointer to ffi_closure_ASM */
    void * toc;			/* TOC */
    void * static_chain;	/* Pointer to closure */
};
#else /* ! __ppc__ && ! __ppc64__ */
#endif
#ifndef X86_WIN64
#define FFI_NATIVE_RAW_API 1	/* x86 has native raw api support */
#endif /* __ppc__ || __ppc64__ */
#endif
#endif

#endif
#if defined (__i386__) || defined(__x86_64__)

#endif /* __i386__ || __x86_64__ */
